@@ -1,3 +1,8 @@
+"""Choatic timeseries generators.
+"""
+# Author: Nathan Trouvain at 2020 <nathan.trouvain@inria.fr>
+# Licence: MIT License
+# Copyright: Xavier Hinaut (2018) <xavier.hinaut@inria.fr>
 import collections
 from typing import Union
 
